#pragma section=".ee_mem"
#define EE_END (u32)__section_end(".ee_mem") //0x08004000

#define PG_LN  0x400 //   
#define PNUM   1     // 

struct NIS
{
 int C_LTS;
 int V_LTS;
 int V_100C;
 u32 crc;
};

const NIS Defval =
{
 20,
 466,
 240,
 0x94C2108F
};

__root __no_init const NIS Storage[PG_LN*PNUM/sizeof(NIS)] @ ".ee_mem";

u32 calc_NIS_crc(const NIS *in_struct)
{
 u32* begin = (u32*)in_struct;
 u32* end   = begin + sizeof(NIS)/sizeof(u32) -1;
 u32* pt;

 u32 tmp     = RCC->AHBENR;
 tmp        |= RCC_AHBENR_CRCEN;
 RCC->AHBENR = tmp;
 tmp        &= ~RCC_AHBENR_CRCEN;
 CRC->CR     =  1; //reset

 for(pt = begin; pt < end; pt++) CRC->DR = *pt;

 u32 tmp2 = CRC->DR;
 RCC->AHBENR = tmp;

 return tmp2;
}

//      -1    
int seek_last_free_ee(void)
{
 u32 *pt = (u32*)Storage;
 u32 i = 0;

 while (pt < (u32*)EE_END)
 {
  if(*pt == (u32)-1) return i;

  pt += sizeof(Storage[0])/sizeof(u32);
  i++;
 }

 return -1;
}

//     
//      -1
int read_ee_num(void)
{
 int retval;

 int k = seek_last_free_ee();

 if(k == 0)        retval =  -1;
 else if(k == -1)  retval = sizeof(Storage)/sizeof(Storage[0]) - 1;
 else              retval = k-1;

 if(retval >= 0)
 {
  u32 crc = calc_NIS_crc(&Storage[retval]);

  if(crc != Storage[retval].crc) retval = -1;
 }

 return retval;
}

//   flash
void save_in_ee(NIS inval)
{
 uint32_t bgn;
 u32 tmp;

 inval.crc = calc_NIS_crc(&inval);

 int k = seek_last_free_ee();

 __disable_interrupt();
 FLASH_Unlock();

 //    
 if(k == -1)
 {
  //  
  for(int i = 0; i<PNUM; i++)
   while(FLASH_ErasePage((uint32_t)(EE_END - PG_LN*i -1))==FLASH_BUSY) continue;

  //     
  k=0;
 }

 bgn = (u32)&Storage[k];
 tmp = (u32)&inval;

 for(int i = 0; i < sizeof(Storage[0])/sizeof(u32); i++)
 {
  while(FLASH_ProgramWord(bgn+(i*sizeof(u32)), *((u32*)(tmp +(i*sizeof(u32))))) ==FLASH_BUSY) continue;
 }

 FLASH_Lock();
 __enable_interrupt();
}


