struct LDF
{
 u32* tcmr; //  output compare (pwm1/pwm2/force level)
 u32  tcmr_shift;
 u32* tccr; //     
 u32* tegr; //event generation   
};

LDF const led_cnl[]=
{
 {
  (u32*) &TIM2->CCMR1,
  0,
  (u32*)&TIM2->CCR1,
  (u32*)&TIM2->EGR
 },
 {
  (u32*) &TIM2->CCMR1,
  8,
  (u32*)&TIM2->CCR2,
  (u32*)&TIM2->EGR
 },
 {
  (u32*) &TIM2->CCMR2,
  0,
  (u32*)&TIM2->CCR3,
  (u32*)&TIM2->EGR
 }
};

enum Led_polarity {led_cc, led_ca};

const int LED_NUM    = sizeof(led_cnl)/sizeof(LDF);
const int offset_cr1 = (s32)((u32)&TIM1->EGR - (u32)&TIM1->CR1) /(s32)sizeof(u32);
const int offset_arr = (s32)((u32)&TIM1->EGR - (u32)&TIM1->ARR) /(s32)sizeof(u32);
const int offset_cer = (s32)((u32)&TIM1->EGR - (u32)&TIM1->CCER)/(s32)sizeof(u32);
const int offset_bdr = (s32)((u32)&TIM1->EGR - (u32)&TIM1->BDTR)/(s32)sizeof(u32);

//    ,   0x100
//       
int Led_brt[LED_NUM];
const u32 Led_deviation[LED_NUM] = {0x100,0x100,0x100};

#define LED_RED_ON do{Led_brt[0] = 0;     Led_brt[1]=0;     Led_brt[2]=0x100;}while(0)
#define LED_GRN_ON do{Led_brt[0] = 0;     Led_brt[1]=0x100; Led_brt[2]=0;    }while(0)
#define LED_BLY_ON do{Led_brt[0] = 0x100; Led_brt[1]=0;     Led_brt[2]=0;    }while(0)

#define LED_RED_OFF do{Led_brt[2] = 0;}while(0)
#define LED_GRN_OFF do{Led_brt[1] = 0;}while(0)
#define LED_BLY_OFF do{Led_brt[0] = 0;}while(0)

void setup_led_tim(int frq = 1000)
{
 for(int i=0;i<LED_NUM;i++)
 {
  *(led_cnl[i].tegr - offset_arr) = SYSTEM_FRQ/8/frq -1;
  *(led_cnl[i].tegr - offset_cer) = TIM_CCER_CC1E|TIM_CCER_CC2E|TIM_CCER_CC3E;//|TIM_CCER_CC4E;
 }
}

//  
void led_processing(const int lst[], bool blink_trg=1, const Led_polarity lpol = led_ca)
{
 int cnt = 0;

 for(int i=0;i<LED_NUM;i++)
 {
  //   -   force active
  *led_cnl[i].tcmr &= ~(ccmr_mask << led_cnl[i].tcmr_shift);

  if(lpol == led_ca) *led_cnl[i].tcmr |= force_active << led_cnl[i].tcmr_shift;
  else               *led_cnl[i].tcmr |= force_inactive << led_cnl[i].tcmr_shift;

  //  
  *(led_cnl[i].tegr - offset_cr1) = 0;

  //   CCR   
  if(lst[i]!=0)
  {
   u32 *pt_arr = (u32*)led_cnl[i].tegr - offset_arr; //  ARR

   //       
   *led_cnl[i].tcmr &= ~(ccmr_mask << led_cnl[i].tcmr_shift);

   //      
   if(lst[i]>0)
   {
    //       
    int pwm_val =  *pt_arr * lst[i] * Led_deviation[i] >>16;

    *led_cnl[i].tcmr &= ~(ccmr_mask << led_cnl[i].tcmr_shift);

    if(cnt==0)
    {
     *led_cnl[i].tccr = pwm_val;

     if(lpol == led_ca) *led_cnl[i].tcmr |= pwm2<<led_cnl[i].tcmr_shift;
     else               *led_cnl[i].tcmr |= pwm1<<led_cnl[i].tcmr_shift;
    }
    else
    {
     *led_cnl[i].tccr = *pt_arr - pwm_val;

     if(lpol == led_ca) *led_cnl[i].tcmr |= pwm1<<led_cnl[i].tcmr_shift;
     else               *led_cnl[i].tcmr |= pwm2<<led_cnl[i].tcmr_shift;

    }
   }
   //       
   else
   {
    if(blink_trg == true)
    {
     //       
     int pwm_val = *pt_arr * (-lst[i]) * Led_deviation[i] >>16;

     if(cnt==0)
     {
      *led_cnl[i].tccr = pwm_val;

     if(lpol == led_ca) *led_cnl[i].tcmr |= pwm2<<led_cnl[i].tcmr_shift;
     else               *led_cnl[i].tcmr |= pwm1<<led_cnl[i].tcmr_shift;
     }
     else
     {
      *led_cnl[i].tccr = *pt_arr - pwm_val;

      if(lpol == led_ca) *led_cnl[i].tcmr |= pwm1<<led_cnl[i].tcmr_shift;
      else               *led_cnl[i].tcmr |= pwm2<<led_cnl[i].tcmr_shift;
     }
    }
   }

   cnt++;
  }

  //update 
  *led_cnl[i].tegr = TIM_EGR_UG;
 }

 //  
 for(int i=0;i<LED_NUM;i++)*(led_cnl[i].tegr - offset_cr1) = TIM_CR1_CEN;
}

//    
//    
//  ,  
void t2scale (int led[], const int t)
{
 const int step = (T_MAX-T_MIN)/(LED_NUM-1);

 //  
 for(int i=0;i<LED_NUM;i++) led[i] = 0;

 //       
 //          
 if     (t <= T_MIN) led[0]         = -0x100;
 else if(t >= T_MAX) led[LED_NUM-1] = -0x100;
 else
 {
  //      
  int Nsc = (t - T_MIN)/step;
  //     
  int delta;

  if(step == 0) delta = t - T_MIN;
  else delta = t - T_MIN - Nsc*step;

  //    0100     
  //    
  led[Nsc+1] = (delta<<8)/step;
  led[Nsc]   = 0x100 - led[Nsc+1];
 }
}

void t_set_2_scale(int led[], const int t)
{
 const int step = (T_MAX-T_MIN)/(LED_NUM-1);

 //  
 for(int i=0;i<LED_NUM;i++) led[i] = 0;

 //       
 //          
 if     (t < T_MIN) led[0]         = -0x100;
 else if(t > T_MAX) led[LED_NUM-1] = -0x100;
 else
 {
  //      
  int Nsc = (t - T_MIN)/step;

  //     
  int delta = t % step;

  //    0100     
  //    
  led[Nsc+1]   = (delta<<8)/step;
  led[Nsc]     = 0x100 - led[Nsc+1];

  if(delta != 0)
  {
   led[Nsc]   = 0x100/2;
   led[Nsc+1] = 0x100/2;
  }
  else led[Nsc] = 0x100/2;
 }
}





