/********************************************************
Workspace: IAR_ARM+SPL
MCU      : STM32f050f4/6
Project  : Tea pot pb controller
Version  :
Date     : 03 2021
*********************************************************/

#include <stm32f0xx.h>
#include <stdbool.h>
#include <uchar.h>
#include <math.h>

#include "stm32f0xx_syscfg.c"
#include "stm32f0xx_gpio.c"
#include "stm32f0xx_dma.c"
#include "stm32f0xx_rcc.c"
#include "stm32f0xx_iwdg.c"
#include "stm32f0xx_pwr.c"
#include "stm32f0xx_flash.c"

#include "bitset.c"
#include "defines.c"
#include "periph_cfg.c"
#include "psu_mon.c"
#include "beep.c"
#include "led_rtns.c"
#include "misc.c"
#include "eep_em.c"
#include "t_sensor_routines.c"
#include "crc_calculation.c"
#include "kb.c"
#include "interrupts.c"

void devcon(void);

void main()
{
 //    
 {
  enable_PVD();
  while(PWR_GetFlagStatus(PWR_FLAG_PVDO)==SET) continue;
  disable_PVD();
 }

 cfg_rcc();
 cfg_gpio();
 cfg_t1();

 setup_led_tim();
 led_processing(Led_brt);
 btn_init();
 cfg_systic();
 generate_sound_table();

 //    
 while(FLASH_OB_UBC()==FLASH_BUSY)continue;
 //      
 set_flash_crc();

 cfg_iwdg();


 __enable_interrupt();


ENDLESS
 {
  // 
  if(!SNSR_TIM) { SNSR_TIM = T_SEC/20; btn_scan(); }

  if(!TMR_TIM) devcon();

  if(Iwdg_systick == true){Iwdg_systick = false; IWDG_ReloadCounter();}
 }
}

void devcon()
{
 static bool itr;
 static enum {start, heater_on, heater_off, error} dev_st = start, dev_st_tmp;
 static enum {
              no_error,
              V_calibration_error,
              V_input_error,
              t_grow_error,
              overheat_error,
              timeout_error,
              Vt_size_error,
              crc_error
             } err;
 static NIS t_param;    //  
 static int t;          // 
 struct                 //  
 {
  bool recal_trg; // 
  bool recal_hi;
  bool recal_lo;
 } static rcs;
 static bool heating_enable;

 //  
 if(get_flash_crc() == false) err = crc_error;

 //   
 {
  static bool trg;

  if(*btn == rls)
  {
   trg = false;

   if(dev_st == heater_on)
   {
    dev_st = heater_off;

    beep(&BEEP_TIM,2000);
   }
  }
  else
  {
   if(trg == false && dev_st == heater_off)
   {
    trg = true;

    dev_st = heater_on;

    beep(&BEEP_TIM,6500);
   }
  }
 }

 if(err != no_error) dev_st = error;
 if(dev_st != dev_st_tmp)
 {
  dev_st_tmp = dev_st;

  itr = 0;
 }

 switch (dev_st)
 {
  case start:
  {
   static u32 l_stage;

   if(!itr)
   {
    l_stage = 0;
    err = no_error;
    heating_enable = false;

    dma_adc_config();
    adc_config();

    //       PA10  
    rcs.recal_trg = check_recal_pin();

    //     rom
    {
     int k = read_ee_num();
     //    
     if(k < 0)
     {
      // 
      rcs.recal_trg = true;
      //   
      t_param = Defval;
     }
     //  
     else
     {
      t_param = Storage[k];
     }
    }

    //          
    if(rcs.recal_trg == true)
    {
     rcs.recal_hi = false;
     rcs.recal_lo = false;
    }
    else
    {
     rcs.recal_hi = true;
     rcs.recal_lo = true;
    }
   }

   // ,    
   if(!LED_TIM)
   {
    LED_TIM = T_SEC/3;

    for(int i = 0; i<sizeof(Led_brt)/sizeof(Led_brt[0]); i++)
    {
     if(i == l_stage) Led_brt [i] = 0x100;
     else             Led_brt [i] = 0;
    }

    l_stage++;

    if(l_stage > sizeof(Led_brt)/sizeof(Led_brt[0]))
    {
     TMR_TIM = T_SEC;

     dev_st = heater_off;
    }
   }
  }
  break;

  case heater_on:
  {
   static u32 heat_off_cntdn; //    , 
   static bool boiling_trg;   //  
   static bool reset_bdetector;

   u32 Vdd;
   static u32 Vin;

   if(!itr)
   {
    boiling_trg = false;
    heating_enable = true;
    reset_bdetector = true;

    heat_off_cntdn = HT_TO;  //    - 8 

    TMR_TIM = T_SEC;
   }

   // 
   if(boiling_trg == false)
   {
    // 
    do_conversion();

    // 
    calc_kVdd(&Vdd);
    calc_Vin(&Vin, Vdd);
    //    
    Vin_2_t(&t, Vin, t_param, rcs.recal_trg);

    //   
    t2scale (Led_brt, t);

    if(!BTD_TIM)
    {
     boiling_trg = boiling_detector(t, &reset_bdetector);
     BTD_TIM = T_SEC;

     //   
     {
      //            
      calc_ambient(&t, Vdd);
      if(t > 85) err = overheat_error;

      //  
      if(!heat_off_cntdn) err = timeout_error;
      else heat_off_cntdn--;

      //          
      if(Vin > VIN_MAX || Vin < VIN_MIN) err = V_input_error;

      //  
      if(rcs.recal_trg == false)
      {
       //   
       if(t > 120) err = overheat_error;
      }
     }
    }
   }
   //   
   else
   {
    heating_enable = false;

    //     ,   
    if(rcs.recal_hi == false)
    {
     t_param.V_100C = Vin;

     rcs.recal_hi = true;
    }

    static int bbs = 0; //    

    bbs++;

    //          
    if(bbs > 3)
    {
     bbs = 0;
     dev_st = heater_off;
    }
    else
    {
     beep(&BEEP_TIM,5500,400);
     TMR_TIM = T_SEC - T_SEC/4;
    }
   }
  }
  break;

  case heater_off:
  {
   static u32 cntdn;
   static int t_prev;

   if(!itr)
   {
    cntdn = 1*60*60;
    t_prev = t;

    heating_enable = false;
   }

   TMR_TIM = T_SEC;

   // 
   do_conversion();

   // 
   u32 Vdd;
   u32 Vin;

   calc_kVdd(&Vdd);
   calc_Vin(&Vin, Vdd);
   Vin_2_t(&t, Vin, t_param, rcs.recal_trg);

   //   
   t2scale (Led_brt, t);

   //   
   {
    //          
    if(Vin > VIN_MAX || Vin < VIN_MIN) err = V_input_error;
   }


   //      
   if(rcs.recal_lo == false)
   {
    if(cntdn) cntdn--;
    //     
    else
    {
     //   
     if(t_prev == t)
     {
      calc_ambient(&t, Vdd);

      t_param.C_LTS = t;
      t_param.V_LTS = Vin;

      rcs.recal_lo = true;
     }

     //   
     cntdn = 1*60*60;
     //  
     t_prev = t;
    }
   }
  }
  break;

  case error:
  {
   static int ind_stage;
   static bool ind_trg;

   if(!itr)
   {
    heating_enable = false;

    ind_stage = 0;
    ind_trg = false;

    LED_RED_OFF;
    LED_GRN_OFF;

    LED_TIM = T_SEC;
   }

   /*
     
        
        2      
   */
   if(!LED_TIM)
   {
    //   
    if(ind_trg == false)
    {
     ind_trg = true;

     // -  
     if(ind_stage == 0)
     {
      LED_TIM = T_SEC;
      ind_stage ++;

      LED_RED_ON;
      LED_GRN_OFF;
     }
     // -     
     else if(ind_stage < (int)err+1)
     {
      LED_TIM = T_SEC/2;
      ind_stage ++;

      LED_GRN_ON;
      beep(&BEEP_TIM, 3000, 300);
     }
     else
     {
      ind_stage = 0;
      ind_trg = false;

      LED_TIM = T_SEC*2;
     }
    }
    //    
    else
    {
     ind_trg = false;
     LED_TIM = T_SEC/2;

     LED_RED_OFF;
     LED_GRN_OFF;
    }
   }

   // 
   switch(err)
   {
    case V_calibration_error:
    {

    }
    break;

    case V_input_error:
    {

    }
    break;

    case t_grow_error:
    {

    }
    break;

    case overheat_error:
    {
     // 
     do_conversion();

     //  
     u32 Vdd;

     calc_kVdd(&Vdd);
     calc_ambient(&t, Vdd);

     if(t < 45)
     {
      err = no_error;
      dev_st = start;
     }
    }
    break;

    case crc_error:
    {

    }
    break;

    case Vt_size_error:
    {

    }
    break;

    default: break;
   }
  }
 }

 itr = 1;

 //    
 if(rcs.recal_trg)
 {
  //  
  if(rcs.recal_hi == true && rcs.recal_lo == true)
  {
   rcs.recal_trg = false;

   save_in_ee(t_param);
   beep(&BEEP_TIM, 1000, 2000);
  }
 }

 led_processing(Led_brt);

 if(heating_enable == true) scr_on();
 else                       scr_off();
}


